// Package gio was automatically generated by github.com/jwijenbergh/puregotk DO NOT EDIT
package gio

import (
	"unsafe"

	"github.com/jwijenbergh/purego"
	"github.com/jwijenbergh/puregotk/internal/core"
	"github.com/jwijenbergh/puregotk/v4/glib"
)

// The virtual function table for #GActionGroup.
type ActionGroupInterface struct {
	GIface uintptr
}

func (x *ActionGroupInterface) GoPointer() uintptr {
	return uintptr(unsafe.Pointer(x))
}

// #GActionGroup represents a group of actions. Actions can be used to
// expose functionality in a structured way, either from one part of a
// program to another, or to the outside world. Action groups are often
// used together with a #GMenuModel that provides additional
// representation data for displaying the actions to the user, e.g. in
// a menu.
//
// The main way to interact with the actions in a GActionGroup is to
// activate them with g_action_group_activate_action(). Activating an
// action may require a #GVariant parameter. The required type of the
// parameter can be inquired with g_action_group_get_action_parameter_type().
// Actions may be disabled, see g_action_group_get_action_enabled().
// Activating a disabled action has no effect.
//
// Actions may optionally have a state in the form of a #GVariant. The
// current state of an action can be inquired with
// g_action_group_get_action_state(). Activating a stateful action may
// change its state, but it is also possible to set the state by calling
// g_action_group_change_action_state().
//
// As typical example, consider a text editing application which has an
// option to change the current font to 'bold'. A good way to represent
// this would be a stateful action, with a boolean state. Activating the
// action would toggle the state.
//
// Each action in the group has a unique name (which is a string).  All
// method calls, except g_action_group_list_actions() take the name of
// an action as an argument.
//
// The #GActionGroup API is meant to be the 'public' API to the action
// group.  The calls here are exactly the interaction that 'external
// forces' (eg: UI, incoming D-Bus messages, etc.) are supposed to have
// with actions.  'Internal' APIs (ie: ones meant only to be accessed by
// the action group implementation) are found on subclasses.  This is
// why you will find - for example - g_action_group_get_action_enabled()
// but not an equivalent set() call.
//
// Signals are emitted on the action group in response to state changes
// on individual actions.
//
// Implementations of #GActionGroup should provide implementations for
// the virtual functions g_action_group_list_actions() and
// g_action_group_query_action().  The other virtual functions should
// not be implemented - their "wrappers" are actually implemented with
// calls to g_action_group_query_action().
type ActionGroup interface {
	GoPointer() uintptr
	SetGoPointer(uintptr)
	ActionAdded(ActionNameVar string)
	ActionEnabledChanged(ActionNameVar string, EnabledVar bool)
	ActionRemoved(ActionNameVar string)
	ActionStateChanged(ActionNameVar string, StateVar *glib.Variant)
	ActivateAction(ActionNameVar string, ParameterVar *glib.Variant)
	ChangeActionState(ActionNameVar string, ValueVar *glib.Variant)
	GetActionEnabled(ActionNameVar string) bool
	GetActionParameterType(ActionNameVar string) *glib.VariantType
	GetActionState(ActionNameVar string) *glib.Variant
	GetActionStateHint(ActionNameVar string) *glib.Variant
	GetActionStateType(ActionNameVar string) *glib.VariantType
	HasAction(ActionNameVar string) bool
	ListActions() uintptr
	QueryAction(ActionNameVar string, EnabledVar bool, ParameterTypeVar **glib.VariantType, StateTypeVar **glib.VariantType, StateHintVar **glib.Variant, StateVar **glib.Variant) bool
}
type ActionGroupBase struct {
	Ptr uintptr
}

func (x *ActionGroupBase) GoPointer() uintptr {
	return x.Ptr
}

func (x *ActionGroupBase) SetGoPointer(ptr uintptr) {
	x.Ptr = ptr
}

// Emits the #GActionGroup::action-added signal on @action_group.
//
// This function should only be called by #GActionGroup implementations.
func (x *ActionGroupBase) ActionAdded(ActionNameVar string) {

	XGActionGroupActionAdded(x.GoPointer(), ActionNameVar)

}

// Emits the #GActionGroup::action-enabled-changed signal on @action_group.
//
// This function should only be called by #GActionGroup implementations.
func (x *ActionGroupBase) ActionEnabledChanged(ActionNameVar string, EnabledVar bool) {

	XGActionGroupActionEnabledChanged(x.GoPointer(), ActionNameVar, EnabledVar)

}

// Emits the #GActionGroup::action-removed signal on @action_group.
//
// This function should only be called by #GActionGroup implementations.
func (x *ActionGroupBase) ActionRemoved(ActionNameVar string) {

	XGActionGroupActionRemoved(x.GoPointer(), ActionNameVar)

}

// Emits the #GActionGroup::action-state-changed signal on @action_group.
//
// This function should only be called by #GActionGroup implementations.
func (x *ActionGroupBase) ActionStateChanged(ActionNameVar string, StateVar *glib.Variant) {

	XGActionGroupActionStateChanged(x.GoPointer(), ActionNameVar, StateVar)

}

// Activate the named action within @action_group.
//
// If the action is expecting a parameter, then the correct type of
// parameter must be given as @parameter.  If the action is expecting no
// parameters then @parameter must be %NULL.  See
// g_action_group_get_action_parameter_type().
//
// If the #GActionGroup implementation supports asynchronous remote
// activation over D-Bus, this call may return before the relevant
// D-Bus traffic has been sent, or any replies have been received. In
// order to block on such asynchronous activation calls,
// g_dbus_connection_flush() should be called prior to the code, which
// depends on the result of the action activation. Without flushing
// the D-Bus connection, there is no guarantee that the action would
// have been activated.
//
// The following code which runs in a remote app instance, shows an
// example of a "quit" action being activated on the primary app
// instance over D-Bus. Here g_dbus_connection_flush() is called
// before `exit()`. Without g_dbus_connection_flush(), the "quit" action
// may fail to be activated on the primary instance.
//
// |[&lt;!-- language="C" --&gt;
// // call "quit" action on primary instance
// g_action_group_activate_action (G_ACTION_GROUP (app), "quit", NULL);
//
// // make sure the action is activated now
// g_dbus_connection_flush (...);
//
// g_debug ("application has been terminated. exiting.");
//
// exit (0);
// ]|
func (x *ActionGroupBase) ActivateAction(ActionNameVar string, ParameterVar *glib.Variant) {

	XGActionGroupActivateAction(x.GoPointer(), ActionNameVar, ParameterVar)

}

// Request for the state of the named action within @action_group to be
// changed to @value.
//
// The action must be stateful and @value must be of the correct type.
// See g_action_group_get_action_state_type().
//
// This call merely requests a change.  The action may refuse to change
// its state or may change its state to something other than @value.
// See g_action_group_get_action_state_hint().
//
// If the @value GVariant is floating, it is consumed.
func (x *ActionGroupBase) ChangeActionState(ActionNameVar string, ValueVar *glib.Variant) {

	XGActionGroupChangeActionState(x.GoPointer(), ActionNameVar, ValueVar)

}

// Checks if the named action within @action_group is currently enabled.
//
// An action must be enabled in order to be activated or in order to
// have its state changed from outside callers.
func (x *ActionGroupBase) GetActionEnabled(ActionNameVar string) bool {

	cret := XGActionGroupGetActionEnabled(x.GoPointer(), ActionNameVar)
	return cret
}

// Queries the type of the parameter that must be given when activating
// the named action within @action_group.
//
// When activating the action using g_action_group_activate_action(),
// the #GVariant given to that function must be of the type returned
// by this function.
//
// In the case that this function returns %NULL, you must not give any
// #GVariant, but %NULL instead.
//
// The parameter type of a particular action will never change but it is
// possible for an action to be removed and for a new action to be added
// with the same name but a different parameter type.
func (x *ActionGroupBase) GetActionParameterType(ActionNameVar string) *glib.VariantType {

	cret := XGActionGroupGetActionParameterType(x.GoPointer(), ActionNameVar)
	return cret
}

// Queries the current state of the named action within @action_group.
//
// If the action is not stateful then %NULL will be returned.  If the
// action is stateful then the type of the return value is the type
// given by g_action_group_get_action_state_type().
//
// The return value (if non-%NULL) should be freed with
// g_variant_unref() when it is no longer required.
func (x *ActionGroupBase) GetActionState(ActionNameVar string) *glib.Variant {

	cret := XGActionGroupGetActionState(x.GoPointer(), ActionNameVar)
	return cret
}

// Requests a hint about the valid range of values for the state of the
// named action within @action_group.
//
// If %NULL is returned it either means that the action is not stateful
// or that there is no hint about the valid range of values for the
// state of the action.
//
// If a #GVariant array is returned then each item in the array is a
// possible value for the state.  If a #GVariant pair (ie: two-tuple) is
// returned then the tuple specifies the inclusive lower and upper bound
// of valid values for the state.
//
// In any case, the information is merely a hint.  It may be possible to
// have a state value outside of the hinted range and setting a value
// within the range may fail.
//
// The return value (if non-%NULL) should be freed with
// g_variant_unref() when it is no longer required.
func (x *ActionGroupBase) GetActionStateHint(ActionNameVar string) *glib.Variant {

	cret := XGActionGroupGetActionStateHint(x.GoPointer(), ActionNameVar)
	return cret
}

// Queries the type of the state of the named action within
// @action_group.
//
// If the action is stateful then this function returns the
// #GVariantType of the state.  All calls to
// g_action_group_change_action_state() must give a #GVariant of this
// type and g_action_group_get_action_state() will return a #GVariant
// of the same type.
//
// If the action is not stateful then this function will return %NULL.
// In that case, g_action_group_get_action_state() will return %NULL
// and you must not call g_action_group_change_action_state().
//
// The state type of a particular action will never change but it is
// possible for an action to be removed and for a new action to be added
// with the same name but a different state type.
func (x *ActionGroupBase) GetActionStateType(ActionNameVar string) *glib.VariantType {

	cret := XGActionGroupGetActionStateType(x.GoPointer(), ActionNameVar)
	return cret
}

// Checks if the named action exists within @action_group.
func (x *ActionGroupBase) HasAction(ActionNameVar string) bool {

	cret := XGActionGroupHasAction(x.GoPointer(), ActionNameVar)
	return cret
}

// Lists the actions contained within @action_group.
//
// The caller is responsible for freeing the list with g_strfreev() when
// it is no longer required.
func (x *ActionGroupBase) ListActions() uintptr {

	cret := XGActionGroupListActions(x.GoPointer())
	return cret
}

// Queries all aspects of the named action within an @action_group.
//
// This function acquires the information available from
// g_action_group_has_action(), g_action_group_get_action_enabled(),
// g_action_group_get_action_parameter_type(),
// g_action_group_get_action_state_type(),
// g_action_group_get_action_state_hint() and
// g_action_group_get_action_state() with a single function call.
//
// This provides two main benefits.
//
// The first is the improvement in efficiency that comes with not having
// to perform repeated lookups of the action in order to discover
// different things about it.  The second is that implementing
// #GActionGroup can now be done by only overriding this one virtual
// function.
//
// The interface provides a default implementation of this function that
// calls the individual functions, as required, to fetch the
// information.  The interface also provides default implementations of
// those functions that call this function.  All implementations,
// therefore, must override either this function or all of the others.
//
// If the action exists, %TRUE is returned and any of the requested
// fields (as indicated by having a non-%NULL reference passed in) are
// filled.  If the action doesn't exist, %FALSE is returned and the
// fields may or may not have been modified.
func (x *ActionGroupBase) QueryAction(ActionNameVar string, EnabledVar bool, ParameterTypeVar **glib.VariantType, StateTypeVar **glib.VariantType, StateHintVar **glib.Variant, StateVar **glib.Variant) bool {

	cret := XGActionGroupQueryAction(x.GoPointer(), ActionNameVar, EnabledVar, ParameterTypeVar, StateTypeVar, StateHintVar, StateVar)
	return cret
}

var XGActionGroupActionAdded func(uintptr, string)
var XGActionGroupActionEnabledChanged func(uintptr, string, bool)
var XGActionGroupActionRemoved func(uintptr, string)
var XGActionGroupActionStateChanged func(uintptr, string, *glib.Variant)
var XGActionGroupActivateAction func(uintptr, string, *glib.Variant)
var XGActionGroupChangeActionState func(uintptr, string, *glib.Variant)
var XGActionGroupGetActionEnabled func(uintptr, string) bool
var XGActionGroupGetActionParameterType func(uintptr, string) *glib.VariantType
var XGActionGroupGetActionState func(uintptr, string) *glib.Variant
var XGActionGroupGetActionStateHint func(uintptr, string) *glib.Variant
var XGActionGroupGetActionStateType func(uintptr, string) *glib.VariantType
var XGActionGroupHasAction func(uintptr, string) bool
var XGActionGroupListActions func(uintptr) uintptr
var XGActionGroupQueryAction func(uintptr, string, bool, **glib.VariantType, **glib.VariantType, **glib.Variant, **glib.Variant) bool

func init() {
	lib, err := purego.Dlopen(core.GetPath("GIO"), purego.RTLD_NOW|purego.RTLD_GLOBAL)
	if err != nil {
		panic(err)
	}

	core.PuregoSafeRegister(&XGActionGroupActionAdded, lib, "g_action_group_action_added")
	core.PuregoSafeRegister(&XGActionGroupActionEnabledChanged, lib, "g_action_group_action_enabled_changed")
	core.PuregoSafeRegister(&XGActionGroupActionRemoved, lib, "g_action_group_action_removed")
	core.PuregoSafeRegister(&XGActionGroupActionStateChanged, lib, "g_action_group_action_state_changed")
	core.PuregoSafeRegister(&XGActionGroupActivateAction, lib, "g_action_group_activate_action")
	core.PuregoSafeRegister(&XGActionGroupChangeActionState, lib, "g_action_group_change_action_state")
	core.PuregoSafeRegister(&XGActionGroupGetActionEnabled, lib, "g_action_group_get_action_enabled")
	core.PuregoSafeRegister(&XGActionGroupGetActionParameterType, lib, "g_action_group_get_action_parameter_type")
	core.PuregoSafeRegister(&XGActionGroupGetActionState, lib, "g_action_group_get_action_state")
	core.PuregoSafeRegister(&XGActionGroupGetActionStateHint, lib, "g_action_group_get_action_state_hint")
	core.PuregoSafeRegister(&XGActionGroupGetActionStateType, lib, "g_action_group_get_action_state_type")
	core.PuregoSafeRegister(&XGActionGroupHasAction, lib, "g_action_group_has_action")
	core.PuregoSafeRegister(&XGActionGroupListActions, lib, "g_action_group_list_actions")
	core.PuregoSafeRegister(&XGActionGroupQueryAction, lib, "g_action_group_query_action")

}
